//-----------------------------------------------------------------------
//	SDRAM_TOP.v
//		Top level module for instantiation of control and FIFOs
//		needed for using the SDRAM on the CaLinx board
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
//	Module:	SDRAM_TOP
//	Desc:	The is the top module containing the SDRAM_CNTRL, the
//		pn_generators and the fifos.
//		Modify any signals internal to this block that you deem
//		necessary. Do not change the interface, PORT names/size,
//		of this module.
//		Read through the code and do as the instructions indicate.
//		This Verilog code is not functional as certain signals
//		have to be generated.
//-----------------------------------------------------------------------

module SDRAM_TOP (
		//-------------------------------------------------------
		//	SDRAM Tristate Databus Control and Data
		//-------------------------------------------------------
		RAM_D_IN, 		// Data in from SDRAM
		RAM_D_OUT,		// Data out to SDRAM
		RAM_D_EN,		// Tristate control
		//-------------------------------------------------------

		//-------------------------------------------------------
		//	SDRAM Control and Address
		//-------------------------------------------------------
		RAM_CLK,		// Clock out to SDRAM
		RAM_CLKE,		// Control to SDRAM
		RAM_DQMH,		// Control to SDRAM
		RAM_DQML,		// Control to SDRAM
		RAM_CS,			// Control to SDRAM
		RAM_RAS,		// Control to SDRAM
		RAM_CAS,		// Control to SDRAM
		RAM_WE,			// Control to SDRAM
		RAM_BA,			// Bank Address to SDRAM
		RAM_A,			// Address to SDRAM
		//-------------------------------------------------------

		//-------------------------------------------------------
		//	Misc.
		//-------------------------------------------------------
		Clock, 			// Clock in from Y5
		Reset,			// Debounced Reset from SW_[1]
		DEBUG, 			// Debug singals to probe pins
		TimeSelect,		// From the dip-switches
		ErrorCount		// Counter of number of errors
		//-------------------------------------------------------
		 );
	
	//---------------------------------------------------------------
	//	SDRAM Tristate Databus Control and Data
	//---------------------------------------------------------------
	input	[31:0]	RAM_D_IN;
	output	[31:0]	RAM_D_OUT;
	output		RAM_D_EN;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	SDRAM Control and Address
	//---------------------------------------------------------------
	output		RAM_CLK;
	output		RAM_CLKE;
	output		RAM_DQMH;
	output		RAM_DQML;
	output		RAM_CS;
	output		RAM_RAS;
	output		RAM_CAS;
	output		RAM_WE;
	output	[1:0]	RAM_BA;
	output	[11:0]	RAM_A;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Misc.
	//---------------------------------------------------------------
	input		Clock;
	input		Reset;
	inout	[31:0]	DEBUG;
	input	[15:0]	TimeSelect;
	output	[22:0]	ErrorCount;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Signals in SDRAM_CNTRL
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Signals in pn_generators
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Signals in FIFOs
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Counts the number of errors
	//---------------------------------------------------------------
	wire		ErrorNext;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Instantiation of your SDRAM_CNTRL module 
	//---------------------------------------------------------------
	sdram_cntrl sdram_cntrl_inst(
		//-------------------------------------------------------
		//	These signals must be present in your sdram_cntrl module
		//-------------------------------------------------------
		RAM_D_EN,		// to SDRAM
		RAM_CLK,		// to SDRAM
		RAM_CLKE,		// to SDRAM
		RAM_DQMH,		// to SDRAM
		RAM_DQML,		// to SDRAM
		RAM_CS,			// to SDRAM
		RAM_RAS,		// to SDRAM
		RAM_CAS,		// to SDRAM
		RAM_WE,			// to SDRAM
		RAM_BA,			// to SDRAM
		RAM_A,			// to SDRAM
		Clock, 			// from FPGA
		Reset,			// from FPGA
		TimeSelect		// from dip-switches
		//-------------------------------------------------------

		//-------------------------------------------------------
		//	You can add as many signals here as you deem
		//	necessary here
		//-------------------------------------------------------
		//-------------------------------------------------------
		 );
	//---------------------------------------------------------------
		 
	//---------------------------------------------------------------
	//	Instantiate your pn-generators here
	//---------------------------------------------------------------
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	instantiate your FIFOs here
	//---------------------------------------------------------------
	// You can assign the sinit signal in the FIFOs to reset
	// assign sinit =		reset;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Error Detection Circuits
	//---------------------------------------------------------------
	//	compare the output of the fifo_2 with the output of the pn_gen2. 
	//	Make sure the pn_gen2 is shifted at the right time.
	//	Increments error counter when the two values are different.
	//	You have to create error_next.
	//	Output the error counter value 
	counter #(23)	ErrorCounter(.OUT(ErrorCount), .CLK(Clock), .ENABLE(ErrorNext), .RESET(Reset));
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Debugging Pins
	//---------------------------------------------------------------
	// This is just an example, most of these signals don't exist.
	//assign 	 DEBUG[31:0]  = {d_in_fifor[12:0], wr_fifor, rd_fifow,wr_fifow, rd_fifor, DEBUG2[14:0]};
	//---------------------------------------------------------------
endmodule   